@@ -22,6 +22,7 @@
 #include "player.hpp"
 
 #include <alsa/asoundlib.h>
+#include <boost/optional.hpp>
 #include <chrono>
 
 
@@ -82,10 +83,10 @@ private:
     std::recursive_mutex mutex_;
     boost::asio::steady_timer timer_;
 
-    std::chrono::microseconds buffer_time_;
-    unsigned int periods_;
+    boost::optional<std::chrono::microseconds> buffer_time_;
+    boost::optional<uint32_t> periods_;
 };
 
 } // namespace player
 
-#endif
+#endif
\ No newline at end of file
