@@ -1,6 +1,6 @@
 /***
     This file is part of snapcast
-    Copyright (C) 2014-2020  Johannes Pohl
+    Copyright (C) 2014-2021  Johannes Pohl
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,9 +16,11 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ***/
 
-#include <assert.h>
+#include <cassert>
 #include <iostream>
 
+#include <pulse/proplist.h>
+
 #include "common/aixlog.hpp"
 #include "common/snap_exception.hpp"
 #include "common/str_compat.hpp"
@@ -31,7 +33,7 @@ using namespace std;
 namespace player
 {
 
-static constexpr std::chrono::milliseconds BUFFER_TIME = 80ms;
+static constexpr std::chrono::milliseconds BUFFER_TIME = 100ms;
 
 static constexpr auto LOG_TAG = "PulsePlayer";
 
@@ -41,7 +43,7 @@ static constexpr auto LOG_TAG = "PulsePlayer";
 // https://www.freedesktop.org/wiki/Software/PulseAudio/Documentation/Developer/Clients/Samples/AsyncPlayback/
 
 
-vector<PcmDevice> PulsePlayer::pcm_list()
+vector<PcmDevice> PulsePlayer::pcm_list(const std::string& parameter)
 {
     auto pa_ml = std::shared_ptr<pa_mainloop>(pa_mainloop_new(), [](pa_mainloop* pa_ml) { pa_mainloop_free(pa_ml); });
     pa_mainloop_api* pa_mlapi = pa_mainloop_get_api(pa_ml.get());
@@ -49,8 +51,14 @@ vector<PcmDevice> PulsePlayer::pcm_list()
         pa_context_disconnect(pa_ctx);
         pa_context_unref(pa_ctx);
     });
-    if (pa_context_connect(pa_ctx.get(), nullptr, PA_CONTEXT_NOFLAGS, nullptr) < 0)
-        throw SnapException("Failed to connect to PulseAudio context: " + std::string(pa_strerror(pa_context_errno(pa_ctx.get()))));
+
+    std::string pa_server;
+    auto params = utils::string::split_pairs(parameter, ',', '=');
+    if (params.find("server") != params.end())
+        pa_server = params["server"];
+
+    if (pa_context_connect(pa_ctx.get(), pa_server.empty() ? nullptr : pa_server.c_str(), PA_CONTEXT_NOFLAGS, nullptr) < 0)
+        throw SnapException("Failed to connect to PulseAudio context, error: " + std::string(pa_strerror(pa_context_errno(pa_ctx.get()))));
 
     static int pa_ready = 0;
     pa_context_set_state_callback(
@@ -82,12 +90,15 @@ vector<PcmDevice> PulsePlayer::pcm_list()
         if (now - wait_start > 5s)
             throw SnapException("Timeout while waiting for PulseAudio to become ready");
         if (pa_mainloop_iterate(pa_ml.get(), 1, nullptr) < 0)
-            throw SnapException("Error while waiting for PulseAudio to become ready: " + std::string(pa_strerror(pa_context_errno(pa_ctx.get()))));
+            throw SnapException("Error while waiting for PulseAudio to become ready, error: " + std::string(pa_strerror(pa_context_errno(pa_ctx.get()))));
         this_thread::sleep_for(1ms);
     }
 
+    if (pa_ready == 2)
+        throw SnapException("PulseAudio context failed, error: " + std::string(pa_strerror(pa_context_errno(pa_ctx.get()))));
+
     static std::vector<PcmDevice> devices;
-    auto op = pa_context_get_sink_info_list(
+    auto* op = pa_context_get_sink_info_list(
         pa_ctx.get(),
         [](pa_context* ctx, const pa_sink_info* i, int eol, void* userdata) mutable {
             std::ignore = ctx;
@@ -99,6 +110,9 @@ vector<PcmDevice> PulsePlayer::pcm_list()
         },
         nullptr);
 
+    if (op == nullptr)
+        throw SnapException("PulseAudio get sink info list failed, error: " + std::string(pa_strerror(pa_context_errno(pa_ctx.get()))));
+
     wait_start = std::chrono::steady_clock::now();
 
     while (pa_operation_get_state(op) != PA_OPERATION_DONE)
@@ -121,13 +135,33 @@ vector<PcmDevice> PulsePlayer::pcm_list()
 
 
 PulsePlayer::PulsePlayer(boost::asio::io_context& io_context, const ClientSettings::Player& settings, std::shared_ptr<Stream> stream)
-    : Player(io_context, settings, stream), latency_(BUFFER_TIME), pa_ml_(nullptr), pa_ctx_(nullptr), playstream_(nullptr)
+    : Player(io_context, settings, stream), latency_(BUFFER_TIME), pa_ml_(nullptr), pa_ctx_(nullptr), playstream_(nullptr), proplist_(nullptr),
+      server_(boost::none)
 {
-    auto params = utils::string::split_pairs(settings.parameter, ',', '=');
+    auto params = utils::string::split_pairs_to_container<std::vector<std::string>>(settings.parameter, ',', '=');
     if (params.find("buffer_time") != params.end())
-        latency_ = std::chrono::milliseconds(std::max(cpt::stoi(params["buffer_time"]), 10));
+        latency_ = std::chrono::milliseconds(std::max(cpt::stoi(params["buffer_time"].front()), 10));
+    if (params.find("server") != params.end())
+        server_ = params["server"].front();
+    properties_[PA_PROP_MEDIA_ROLE] = "music";
+    properties_[PA_PROP_APPLICATION_ICON_NAME] = "snapcast";
+    if (params.find("property") != params.end())
+    {
+        for (const auto& p : params["property"])
+        {
+            std::string value;
+            std::string key = utils::string::split_left(p, '=', value);
+            if (!key.empty())
+                properties_[key] = value;
+        }
+    }
+    for (const auto& property : properties_)
+    {
+        if (!property.second.empty())
+            LOG(INFO, LOG_TAG) << "Setting property \"" << property.first << "\" to \"" << property.second << "\"\n";
+    }
 
-    LOG(INFO, LOG_TAG) << "Using buffer_time: " << latency_.count() / 1000 << " ms\n";
+    LOG(INFO, LOG_TAG) << "Using buffer_time: " << latency_.count() / 1000 << " ms, server: " << server_.value_or("default") << "\n";
 }
 
 
@@ -180,12 +214,12 @@ void PulsePlayer::triggerVolumeUpdate()
         [](pa_context* ctx, const pa_sink_input_info* info, int eol, void* userdata) {
             std::ignore = ctx;
             LOG(DEBUG, LOG_TAG) << "pa_context_get_sink_info_by_index info: " << (info != nullptr) << ", eol: " << eol << "\n";
-            if (info)
+            if (info != nullptr)
             {
-                auto self = static_cast<PulsePlayer*>(userdata);
-                auto volume = (double)pa_cvolume_avg(&(info->volume)) / (double)PA_VOLUME_NORM;
-                bool muted = (info->mute != 0);
-                LOG(DEBUG, LOG_TAG) << "volume changed: " << volume << ", muted: " << muted << "\n";
+                auto* self = static_cast<PulsePlayer*>(userdata);
+                self->volume_ = static_cast<double>(pa_cvolume_avg(&(info->volume))) / static_cast<double>(PA_VOLUME_NORM);
+                self->muted_ = (info->mute != 0);
+                LOG(DEBUG, LOG_TAG) << "Volume changed: " << self->volume_ << ", muted: " << self->muted_ << "\n";
 
                 auto now = std::chrono::steady_clock::now();
                 if (now - self->last_change_ < 1s)
@@ -195,7 +229,7 @@ void PulsePlayer::triggerVolumeUpdate()
                                         << " ms => ignoring volume change\n";
                     return;
                 }
-                self->notifyVolumeChange(volume, muted);
+                self->notifyVolumeChange(self->volume_, self->muted_);
             }
         },
         this);
@@ -211,7 +245,7 @@ void PulsePlayer::subscribeCallback(pa_context* ctx, pa_subscription_event_type_
     if (facility == PA_SUBSCRIPTION_EVENT_SINK_INPUT)
     {
         LOG(DEBUG, LOG_TAG) << "event_type: " << event_type << ", facility: " << facility << "\n";
-        if (playstream_ && (idx == pa_stream_get_index(playstream_)))
+        if ((playstream_ != nullptr) && (idx == pa_stream_get_index(playstream_)))
             triggerVolumeUpdate();
     }
 }
@@ -285,10 +319,9 @@ void PulsePlayer::writeCallback(pa_stream* stream, size_t nbytes)
     if (buffer_.size() < nbytes)
         buffer_.resize(nbytes);
     // LOG(TRACE, LOG_TAG) << "writeCallback latency " << usec << " us, frames: " << numFrames << "\n";
-    if (!stream_->getPlayerChunk(buffer_.data(), std::chrono::microseconds(usec), numFrames))
+    if (!stream_->getPlayerChunkOrSilence(buffer_.data(), std::chrono::microseconds(usec), numFrames))
     {
-        // LOG(INFO, LOG_TAG) << "Failed to get chunk. Playing silence.\n";
-        memset(buffer_.data(), 0, numFrames);
+        // LOG(TRACE, LOG_TAG) << "Failed to get chunk. Playing silence.\n";
     }
     else
     {
@@ -318,15 +351,25 @@ void PulsePlayer::start()
     else if (format.bits() == 32)
         pa_ss_.format = PA_SAMPLE_S32LE;
     else
-        throw SnapException("Unsupported sample format: " + cpt::to_string(format.bits()));
+        throw SnapException("Unsupported sample format \"" + cpt::to_string(format.bits()) + "\"");
 
     // Create a mainloop API and connection to the default server
     pa_ready_ = 0;
     pa_ml_ = pa_mainloop_new();
     pa_mainloop_api* pa_mlapi = pa_mainloop_get_api(pa_ml_);
-    pa_ctx_ = pa_context_new(pa_mlapi, "Snapcast");
-    if (pa_context_connect(pa_ctx_, nullptr, PA_CONTEXT_NOFLAGS, nullptr) < 0)
-        throw SnapException("Failed to connect to PulseAudio context: " + std::string(pa_strerror(pa_context_errno(pa_ctx_))));
+
+    proplist_ = pa_proplist_new();
+    for (const auto& property : properties_)
+    {
+        if (!property.second.empty())
+            pa_proplist_sets(proplist_, property.first.c_str(), property.second.c_str());
+    }
+
+    pa_ctx_ = pa_context_new_with_proplist(pa_mlapi, "Snapcast", proplist_);
+
+    const char* server = server_.has_value() ? server_.value().c_str() : nullptr;
+    if (pa_context_connect(pa_ctx_, server, PA_CONTEXT_NOFLAGS, nullptr) < 0)
+        throw SnapException("Failed to connect to PulseAudio context, error: " + std::string(pa_strerror(pa_context_errno(pa_ctx_))));
 
     // This function defines a callback so the server will tell us it's state.
     // Our callback will wait for the state to be ready.  The callback will
@@ -336,7 +379,7 @@ void PulsePlayer::start()
     pa_context_set_state_callback(
         pa_ctx_,
         [](pa_context* c, void* userdata) {
-            auto self = static_cast<PulsePlayer*>(userdata);
+            auto* self = static_cast<PulsePlayer*>(userdata);
             self->stateCallback(c);
         },
         this);
@@ -350,15 +393,15 @@ void PulsePlayer::start()
         if (now - wait_start > 5s)
             throw SnapException("Timeout while waiting for PulseAudio to become ready");
         if (pa_mainloop_iterate(pa_ml_, 1, nullptr) < 0)
-            throw SnapException("Error while waiting for PulseAudio to become ready: " + std::string(pa_strerror(pa_context_errno(pa_ctx_))));
+            throw SnapException("Error while waiting for PulseAudio to become ready, error: " + std::string(pa_strerror(pa_context_errno(pa_ctx_))));
         this_thread::sleep_for(1ms);
     }
 
     if (pa_ready_ == 2)
-        throw SnapException("PulseAudio is not ready");
+        throw SnapException("PulseAudio is not ready, error: " + std::string(pa_strerror(pa_context_errno(pa_ctx_))));
 
     playstream_ = pa_stream_new(pa_ctx_, "Playback", &pa_ss_, nullptr);
-    if (!playstream_)
+    if (playstream_ == nullptr)
         throw SnapException("Failed to create PulseAudio stream");
 
     if (settings_.mixer.mode == ClientSettings::Mixer::Mode::hardware)
@@ -366,19 +409,19 @@ void PulsePlayer::start()
         pa_context_set_subscribe_callback(
             pa_ctx_,
             [](pa_context* ctx, pa_subscription_event_type_t event_type, uint32_t idx, void* userdata) {
-                auto self = static_cast<PulsePlayer*>(userdata);
+                auto* self = static_cast<PulsePlayer*>(userdata);
                 self->subscribeCallback(ctx, event_type, idx);
             },
             this);
-        const pa_subscription_mask_t mask = static_cast<pa_subscription_mask_t>(PA_SUBSCRIPTION_MASK_SINK_INPUT);
+        const auto mask = static_cast<pa_subscription_mask_t>(PA_SUBSCRIPTION_MASK_SINK_INPUT);
 
         pa_context_subscribe(
             pa_ctx_, mask,
             [](pa_context* ctx, int success, void* userdata) {
                 std::ignore = ctx;
-                if (success)
+                if (success != 0)
                 {
-                    auto self = static_cast<PulsePlayer*>(userdata);
+                    auto* self = static_cast<PulsePlayer*>(userdata);
                     self->triggerVolumeUpdate();
                 }
             },
@@ -388,7 +431,7 @@ void PulsePlayer::start()
     pa_stream_set_write_callback(
         playstream_,
         [](pa_stream* stream, size_t length, void* userdata) {
-            auto self = static_cast<PulsePlayer*>(userdata);
+            auto* self = static_cast<PulsePlayer*>(userdata);
             self->writeCallback(stream, length);
         },
         this);
@@ -396,7 +439,7 @@ void PulsePlayer::start()
     pa_stream_set_underflow_callback(
         playstream_,
         [](pa_stream* stream, void* userdata) {
-            auto self = static_cast<PulsePlayer*>(userdata);
+            auto* self = static_cast<PulsePlayer*>(userdata);
             self->underflowCallback(stream);
         },
         this);
@@ -430,27 +473,27 @@ void PulsePlayer::start()
 void PulsePlayer::stop()
 {
     LOG(INFO, LOG_TAG) << "Stop\n";
-    if (pa_ml_)
+    if (pa_ml_ != nullptr)
     {
         pa_mainloop_quit(pa_ml_, 0);
     }
 
     Player::stop();
 
-    if (pa_ctx_)
+    if (pa_ctx_ != nullptr)
     {
         pa_context_disconnect(pa_ctx_);
         pa_context_unref(pa_ctx_);
         pa_ctx_ = nullptr;
     }
 
-    if (pa_ml_)
+    if (pa_ml_ != nullptr)
     {
         pa_mainloop_free(pa_ml_);
         pa_ml_ = nullptr;
     }
 
-    if (playstream_)
+    if (playstream_ != nullptr)
     {
         pa_stream_set_state_callback(playstream_, nullptr, nullptr);
         pa_stream_set_read_callback(playstream_, nullptr, nullptr);
@@ -461,6 +504,12 @@ void PulsePlayer::stop()
         pa_stream_unref(playstream_);
         playstream_ = nullptr;
     }
+
+    if (proplist_ != nullptr)
+    {
+        pa_proplist_free(proplist_);
+        proplist_ = nullptr;
+    }
 }
 
 } // namespace player
