@@ -1,6 +1,6 @@
 /***
     This file is part of snapcast
-    Copyright (C) 2014-2020  Johannes Pohl
+    Copyright (C) 2014-2021  Johannes Pohl
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
 #include <chrono>
 #include <iostream>
 #ifndef WINDOWS
-#include <signal.h>
+#include <csignal>
 #include <sys/resource.h>
 #endif
 
@@ -44,6 +44,7 @@
 #include "common/snap_exception.hpp"
 #include "common/str_compat.hpp"
 #include "common/utils.hpp"
+#include "common/version.hpp"
 #include "metadata.hpp"
 
 
@@ -55,8 +56,11 @@ using namespace std::chrono_literals;
 
 static constexpr auto LOG_TAG = "Snapclient";
 
-PcmDevice getPcmDevice(const std::string& player, const std::string& soundcard)
+
+PcmDevice getPcmDevice(const std::string& player, const std::string& parameter, const std::string& soundcard)
 {
+    LOG(DEBUG, LOG_TAG) << "Trying to get PCM device for player: " << player << ", parameter: "
+                        << ", card: " << soundcard << "\n";
 #if defined(HAS_ALSA) || defined(HAS_PULSE) || defined(HAS_WASAPI)
     vector<PcmDevice> pcm_devices;
 #if defined(HAS_ALSA)
@@ -65,12 +69,14 @@ PcmDevice getPcmDevice(const std::string& player, const std::string& soundcard)
 #endif
 #if defined(HAS_PULSE)
     if (player == player::PULSE)
-        pcm_devices = PulsePlayer::pcm_list();
+        pcm_devices = PulsePlayer::pcm_list(parameter);
 #endif
 #if defined(HAS_WASAPI)
     if (player == player::WASAPI)
         pcm_devices = WASAPIPlayer::pcm_list();
 #endif
+    if (player == player::FILE)
+        return FilePlayer::pcm_list(parameter).front();
     try
     {
         int soundcardIdx = cpt::stoi(soundcard);
@@ -87,6 +93,7 @@ PcmDevice getPcmDevice(const std::string& player, const std::string& soundcard)
             return dev;
 #endif
     std::ignore = player;
+    std::ignore = parameter;
     PcmDevice pcm_device;
     pcm_device.name = soundcard;
     return pcm_device;
@@ -121,7 +128,7 @@ int main(int argc, char** argv)
     int exitcode = EXIT_SUCCESS;
     try
     {
-        string meta_script("");
+        string meta_script;
         ClientSettings settings;
         string pcm_device(player::DEFAULT_DEVICE);
 
@@ -194,12 +201,12 @@ int main(int argc, char** argv)
 
         if (versionSwitch->is_set())
         {
-            cout << "snapclient v" << VERSION << "\n"
-                 << "Copyright (C) 2014-2020 BadAix (snapcast@badaix.de).\n"
+            cout << "snapclient v" << version::code << (!version::rev().empty() ? (" (rev " + version::rev(8) + ")") : ("")) << "\n"
+                 << "Copyright (C) 2014-2021 BadAix (snapcast@badaix.de).\n"
                  << "License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>.\n"
                  << "This is free software: you are free to change and redistribute it.\n"
                  << "There is NO WARRANTY, to the extent permitted by law.\n\n"
-                 << "Written by Johannes M. Pohl.\n\n";
+                 << "Written by Johannes M. Pohl and contributors <https://github.com/badaix/snapcast/graphs/contributors>.\n\n";
             exit(EXIT_SUCCESS);
         }
 
@@ -208,30 +215,37 @@ int main(int argc, char** argv)
 #if defined(HAS_ALSA) || defined(HAS_PULSE) || defined(HAS_WASAPI)
         if (listSwitch->is_set())
         {
-            vector<PcmDevice> pcm_devices;
+            try
+            {
+                vector<PcmDevice> pcm_devices;
 #if defined(HAS_ALSA)
-            if (settings.player.player_name == player::ALSA)
-                pcm_devices = AlsaPlayer::pcm_list();
+                if (settings.player.player_name == player::ALSA)
+                    pcm_devices = AlsaPlayer::pcm_list();
 #endif
 #if defined(HAS_PULSE)
-            if (settings.player.player_name == player::PULSE)
-                pcm_devices = PulsePlayer::pcm_list();
+                if (settings.player.player_name == player::PULSE)
+                    pcm_devices = PulsePlayer::pcm_list(settings.player.parameter);
 #endif
 #if defined(HAS_WASAPI)
-            if (settings.player.player_name == player::WASAPI)
-                pcm_devices = WASAPIPlayer::pcm_list();
+                if (settings.player.player_name == player::WASAPI)
+                    pcm_devices = WASAPIPlayer::pcm_list();
 #endif
 #ifdef WINDOWS
-            // Set console code page to UTF-8 so console known how to interpret string data
-            SetConsoleOutputCP(CP_UTF8);
-            // Enable buffering to prevent VS from chopping up UTF-8 byte sequences
-            setvbuf(stdout, nullptr, _IOFBF, 1000);
+                // Set console code page to UTF-8 so console known how to interpret string data
+                SetConsoleOutputCP(CP_UTF8);
+                // Enable buffering to prevent VS from chopping up UTF-8 byte sequences
+                setvbuf(stdout, nullptr, _IOFBF, 1000);
 #endif
-            for (const auto& dev : pcm_devices)
-                cout << dev.idx << ": " << dev.name << "\n" << dev.description << "\n\n";
+                for (const auto& dev : pcm_devices)
+                    cout << dev.idx << ": " << dev.name << "\n" << dev.description << "\n\n";
 
-            if (pcm_devices.empty())
-                cout << "No PCM device available for audio backend \"" << settings.player.player_name << "\"\n";
+                if (pcm_devices.empty())
+                    cout << "No PCM device available for audio backend \"" << settings.player.player_name << "\"\n";
+            }
+            catch (const std::exception& e)
+            {
+                cout << "Failed to get device list: " << e.what() << "\n";
+            }
             exit(EXIT_SUCCESS);
         }
 #endif
@@ -274,7 +288,7 @@ int main(int argc, char** argv)
         string logformat = "%Y-%m-%d %H-%M-%S.#ms [#severity] (#tag_func)";
         if (settings.logging.sink.find("file:") != string::npos)
         {
-            string logfile = settings.logging.sink.substr(settings.logging.sink.find(":") + 1);
+            string logfile = settings.logging.sink.substr(settings.logging.sink.find(':') + 1);
             AixLog::Log::init<AixLog::SinkFile>(logfilter, logfile, logformat);
         }
         else if (settings.logging.sink == "stdout")
@@ -301,13 +315,13 @@ int main(int argc, char** argv)
             string pidFile = "/var/run/snapclient/pid";
             if (settings.instance != 1)
                 pidFile += "." + cpt::to_string(settings.instance);
-            string user = "";
-            string group = "";
+            string user;
+            string group;
 
             if (userValue->is_set())
             {
                 if (userValue->value().empty())
-                    std::invalid_argument("user must not be empty");
+                    throw std::invalid_argument("user must not be empty");
 
                 vector<string> user_group = utils::string::split(userValue->value(), ':');
                 user = user_group[0];
@@ -324,15 +338,6 @@ int main(int argc, char** argv)
         }
 #endif
 
-        settings.player.pcm_device = getPcmDevice(settings.player.player_name, pcm_device);
-#if defined(HAS_ALSA)
-        if (settings.player.pcm_device.idx == -1)
-        {
-            cout << "PCM device \"" << pcm_device << "\" not found\n";
-            //			exit(EXIT_FAILURE);
-        }
-#endif
-
 #ifdef HAS_SOXR
         if (sample_format->is_set())
         {
@@ -361,7 +366,9 @@ int main(int argc, char** argv)
             else if (settings.player.player_name == player::PULSE)
             {
                 cout << "Options are a comma separated list of:\n"
-                     << " \"buffer_time=<buffer size [ms]>\" - default 80, min 10\n";
+                     << " \"buffer_time=<buffer size [ms]>\" - default 100, min 10\n"
+                     << " \"server=<PulseAudio server>\" - default not-set: use the default server\n"
+                     << " \"property=<key>=<value>\" - can be set multiple times, default 'media.role=music'\n";
             }
 #endif
 #ifdef HAS_ALSA
@@ -379,6 +386,15 @@ int main(int argc, char** argv)
             exit(EXIT_SUCCESS);
         }
 
+        settings.player.pcm_device = getPcmDevice(settings.player.player_name, settings.player.parameter, pcm_device);
+#if defined(HAS_ALSA)
+        if (settings.player.pcm_device.idx == -1)
+        {
+            LOG(ERROR, LOG_TAG) << "PCM device \"" << pcm_device << "\" not found\n";
+            // exit(EXIT_FAILURE);
+        }
+#endif
+
         string mode = utils::string::split_left(mixer_mode->value(), ':', settings.player.mixer.parameter);
         if (mode == "software")
             settings.player.mixer.mode = ClientSettings::Mixer::Mode::software;
@@ -410,6 +426,8 @@ int main(int argc, char** argv)
             io_context.stop();
         });
 
+        LOG(INFO, LOG_TAG) << "Version " << version::code << (!version::rev().empty() ? (", revision " + version::rev(8)) : ("")) << "\n";
+
         // Setup metadata handling
         auto meta(metaStderr ? std::make_unique<MetaStderrAdapter>() : std::make_unique<MetadataAdapter>());
         auto controller = make_shared<Controller>(io_context, settings, std::move(meta));
